///////////////////////////////////////////////////////////////////////
//
// HPVersioner.cpp: 
//
// Copyright 2000-2001, Hewlett-Packard Corp.
//
// Created 12/21/99 (by hh)
//
///////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <stdio.h>
#include <fcntl.h>
#include <time.h>
#include <string.h>
#include <errno.h>
#include <string>
using namespace std;
#include "HPVersionInfo.h"

#define secsPerDay (60*60*24)
//
// "First build day" defined here:
//
enum 
{
	defFirstDay = 1,
	defFirstMonth = 1,
	defFirstYear = 101		//Year - 1900
};
//These are globals because they get set in a subroutine
int firstDay = defFirstDay;
int firstMonth = defFirstMonth;
int firstYear = defFirstYear;


char* NextQuote(char* pch)
{
    while (*pch && (*pch != '\"'))
        pch++;
    if (*pch)
        return pch;
    return NULL;
}

void WriteVer(char* szVerLine, CVersionInfo* pv)
{
    char* pch = strstr(szVerLine, "VALUE");

    //Find beginning of attribute name
    if (!(pch = NextQuote(pch)))
        return;
    //Find end of attribute name
    if (!(pch = NextQuote(++pch)))
        return;
    //Find beginning of attribute value
    if (!(pch = NextQuote(++pch)))
        return;
    //Print ver info:
    pv->AsResource(++pch);
    strcat(pch, "\\0\"\r\n");
}

void WriteVerRaw(char* szVerLine, 
				 CVersionInfo* pv, 
				 char* pchBegin)
{
	char szFind[] = "VERSION ";
	char* pch = strstr(pchBegin, szFind) + sizeof(szFind) - 1;

	pv->AsResource(pch);
	strcat(pch, "\r\n");
}

void GetEnvVar(LPCTSTR lpName, int* pi)
{
	TCHAR tszEnvData[32];
	UINT cb;

	cb = GetEnvironmentVariable(lpName, 
								tszEnvData, sizeof(tszEnvData));
	if (cb)
	{
		if (sizeof(tszEnvData) < cb)
		{
			//Variable too big, not read
			printf("WARNING, could not read %s, too much data\r\n", 
						lpName);
			return;
		}
		*pi = atoi(tszEnvData);
	}
}

void GetEnvData()
{
	GetEnvVar("HPVERFIRSTDAY", &firstDay);
	GetEnvVar("HPVERFIRSTMONTH", &firstMonth);
	GetEnvVar("HPVERFIRSTYEAR", &firstYear);
}

void PrintHeader(int verNow)
{
	CVersionInfo verOut(GetModuleHandle(NULL));
	string strVer;

	printf("%s's %s\r\n", verOut.GetCompanyName(), 
						verOut.GetExtProductName(&strVer));
	printf("%s\r\n\r\n", verOut.GetLegalInfo());

	printf("Usage: HPVersioner [src resfile] [dst resfile] [version string (maj.min)]\r\n");
	printf(" (Todays build: %d)\r\n", verNow);
}

int main(int argc, char* argv[])
{
    FILE* fSrc = NULL, *fDst = NULL;
    struct tm firstBldDay;
    time_t dayZero, dayNow;
    unsigned short verNow = 1;
    CVersionInfo ver;
    char szBuf[1024];

    int nret = 0;

	//See if there is anything in the environment for us
	GetEnvData();
    //Calculate 'verNow'
    memset(&firstBldDay, 0, sizeof(struct tm));
    firstBldDay.tm_hour = 1;
    firstBldDay.tm_mday = firstDay;
    firstBldDay.tm_mon = firstMonth;
    firstBldDay.tm_year = firstYear;
    dayZero = mktime(&firstBldDay);
    dayNow = time(NULL);
    //this check just in case:
    if (dayZero < dayNow)
        verNow = (dayNow - dayZero) / secsPerDay;

    if (argc < 4)
    {
		PrintHeader(verNow);
        return nret;
    }
	//Make sure default mode is binary
	_fmode = _O_BINARY;
    fSrc = fopen(argv[1], "r");
	if (fSrc)
	    fDst = fopen(argv[2], "w");
    if (fSrc && fDst)
    {
		char* pch;

		ver.SetFromString(argv[3]);
		ver.SetMaintenance(verNow);

        while (!feof(fSrc))
        {
            if (fgets(szBuf, 1024, fSrc))
            {
                if (strstr(szBuf, "FileVersion") || 
							strstr(szBuf, "ProductVersion"))
                    WriteVer(szBuf, &ver);
				else if ((pch = strstr(szBuf, "FILEVERSION")) || 
							(pch = strstr(szBuf, "PRODUCTVERSION")))
                    WriteVerRaw(szBuf, &ver, pch);
                if (EOF == fputs(szBuf, fDst))
                {
                    nret = 2;
                    break;
                }
            }
            else
            {
                nret = errno;
                break;
            }
        }
    }
    if (NULL == fSrc)
    {
        printf("Could not read from %s\r\n", argv[1]);
        if (!nret)
            nret = errno;
    }
    else
        fclose(fSrc);
    if (NULL == fDst)
    {
		if (fSrc)
	        printf("Could not write to %s\r\n", argv[2]);
        if (!nret)
            nret = errno;
    }
    else
	{
		if (fSrc)
			printf("%s written with version %s", 
						argv[2], ver.AsString());
        fclose(fDst);
	}
    return nret;
}
